package planet;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import figure.Point3D;
import figure.SphericalAngle;

//--------------------------------------------------------------------//
/**
 * ff[^
 */
public class Planet implements Cloneable{

/** z\ */
  public static final int SUN=0;

/** f\ */
  public static final int INNER_PLANET=1;

/** Of\ */
  public static final int OUTER_PLANET=2;

/** O_̐ */
  public static final int ORBIT_VIEW_COUNT=360;

/** EX */
  protected double epoch=2451545.0;

/** f̖O */
  protected String name;

/** \F */
  protected Color color;

/** \a */
  protected int radius;

/** Oa AU */
  protected double sma;

/** S */
  protected double eccen;

/** ߓ_o(rad) */
  protected double lofp;

/** _o(rad) */
  protected double lofan;

/** OXΊp(rad) */
  protected double inc;

/** ωo(rad) */
  protected double meanl;

/** px(rad/) */
  protected double pinv;

/** f^Cv */
  protected int pltype;

/** fʒu */
  protected Point3D p;

/** fʏWʒu */
  protected ViewPoint3D vp;

/**
 * ff[^\z
 * @param name f̖O
 * @param color \F
 * @param radius \a
 * @param sma Oa
 * @param eccen S
 * @param lofp ߓ_o
 * @param lofan _o
 * @param inc OXΊp
 * @param meanl ωo
 * @param pinv px
 */
  public Planet(String name,Color color,int radius,double sma,
      	      	double eccen,double lofp,double lofan,double inc,
      	      	double meanl,double pinv){
    this.name=name;
    this.color=color;
    this.radius=radius;
    this.sma=sma;
    this.eccen=eccen;
    this.lofp=lofp;
    this.lofan=lofan;
    this.inc=inc;
    this.meanl=meanl;
    this.pinv=pinv;
    pltype=SUN;
    p=new Point3D();
    vp=new ViewPoint3D();
  }

//--------------------------------------------------------------------//
/**
 * OԂ
 * @return O
 */
  public String getName(){
    return name;
  }

//--------------------------------------------------------------------//
/**
 * \FԂ
 * @return \F
 */
  public Color getColor(){
    return color;
  }

//--------------------------------------------------------------------//
/**
 * fʒuԂ
 * @return fʒu\_
 */
  public Point3D getLocation(){
    return p;
  }

//--------------------------------------------------------------------//
/**
 * fʒuԂ
 * @return fʒu\_
 */
  public ViewPoint3D getViewPoint(){
    return vp;
  }

//--------------------------------------------------------------------//
/**
 * _Ԃ<br>
 * _ = Oa~( 1 + S)
 * @return _
 */
  public double getMaxRadius(){
    return sma*(1.0+eccen);
  }

//--------------------------------------------------------------------//
/**
 * ߓ_Ԃ<br>
 * ߓ_ = Oa~( 1 - S)
 * @return ߓ_
 */
  public double getMinRadius(){
    return sma*(1.0-eccen);
  }

//--------------------------------------------------------------------//
/**
 * fWʒuvZ
 * @param julianDay EX
 */
  public void calculateCoordinates(double julianDay){
    double m=meanl-lofp+pinv*(julianDay-epoch);  // ϋߓ_p
    double eeh=0.5*KeplerFunction.eccentricAnomaly(m,eccen);
    double v=Math.PI;
    if(Math.cos(eeh)!=0.0){
      double tanv=Math.sqrt((1.0+eccen)/(1.0-eccen))*Math.tan(eeh);
      v=2.0*Math.atan(tanv);
    }
    v+=lofp;
    double lens=sma*(1.0-eccen*Math.cos(2.0*eeh));
    double ss=Math.sin(v-lofan);
    double cinc=1.0-Math.cos(inc);
    double x=lens*(Math.cos(v)+ss*Math.sin(lofan)*cinc);
    double y=lens*(Math.sin(v)-ss*Math.cos(lofan)*cinc);
    double z=lens*ss*Math.sin(inc);
    p.setLocation(x,y,z);
  }

//--------------------------------------------------------------------//
/**
 * f\ʒuݒ肷
 * @param center plS_
 * @param rcoef 1AU ̃sNZ
 * @param theta _px
 */
  public void setViewPosition(Point center,double rcoef,
      	      	      	      SphericalAngle theta){
    Point3D tvp=(Point3D)p.clone();
    theta.rotate(tvp);
    vp.setLocation((int)(rcoef*tvp.x)+center.x,
      	      	   (int)(rcoef*tvp.y)+center.y,
      	      	   (int)(rcoef*tvp.z));
  }

//--------------------------------------------------------------------//
/**
 * f^Cv𔻕ʂ
 * @param thres fEOf臒l
 */
  public void setType(double thres){
    if(sma==0.0){           // z
      pltype=SUN;
    }
    else if(sma<=thres){    // f
      pltype=INNER_PLANET;
    }
    else{                   // Of
      pltype=OUTER_PLANET;
    }
  }

//--------------------------------------------------------------------//
/**
 * EXݒ肷
 * @param epoch EX
 */
  public void setEpoch(double epoch){
    this.epoch=epoch;
  }

//--------------------------------------------------------------------//
/**
 * fNbN𔻒肷
 * @param x }EXʒu<i>x</i>W
 * @param y }EXʒu<i>y</i>W
 * @param viewMode \[h
 * @return }EXʒuf\͈͓ɂꍇ true
 */
  public boolean clickPlanet(int x,int y,int viewMode){
    if((pltype==SUN) || (pltype==viewMode)){
      int vx=vp.getX();
      int vy=vp.getY();
      int dist=(vx-x)*(vx-x)+(vy-y)*(vy-y);
      return (dist<=radius*radius);
    }
    else{
      return false;
    }
  }

//--------------------------------------------------------------------//
/**
 * f`悷
 * @param g `悷OtBbNXReLXg
 * @param viewMode \[h
 * @param center plS_
 * @param rcoef 1AU ̃sNZ
 * @param theta _px
 * @param orbitColor O`F
 */
  public void drawPlanet(Graphics g,int viewMode,Point center,
      	      	      	 double rcoef,SphericalAngle theta,
      	      	      	 Color orbitColor){
    if((pltype==SUN) || (pltype==viewMode)){
      g.setColor(orbitColor);
      drawOrbit(g,center,rcoef,theta);
      g.setColor(color);
      g.fillOval(vp.getX()-radius,vp.getY()-radius,2*radius,2*radius);
    }
  }

//--------------------------------------------------------------------//
/**
 * O`悷
 * @param g `悷OtBbNXReLXg
 * @param center plS_
 * @param rcoef 1AU ̃sNZ
 * @param theta _px
 */
  protected void drawOrbit(Graphics g,Point center,double rcoef,
      	      	      	   SphericalAngle theta){
// O_W
    int ox[]=new int[ORBIT_VIEW_COUNT];
    int oy[]=new int[ORBIT_VIEW_COUNT];

    double ccpa=Math.cos(lofp-lofan);
    double sspa=Math.sin(lofp-lofan);
    double cinc=1.0-Math.cos(inc);
    double sinc=Math.sin(inc);
    double ccan=Math.cos(lofan)*cinc;
    double ssan=Math.sin(lofan)*cinc;
    double m11=Math.cos(lofp)+sspa*ssan;
    double m12=-Math.sin(lofp)+ccpa*ssan;
    double m21=Math.sin(lofp)-sspa*ccan;
    double m22=Math.cos(lofp)-ccpa*ccan;
    double m31=sspa*sinc;
    double m32=ccpa*sinc;
    double smaY=sma*Math.sqrt(1.0-eccen*eccen);
    Point3D vp=new Point3D();
    double cc=1.0;
    double ss=0.0;
    double dtheta=2.0*Math.PI/ox.length;
    double dc=Math.cos(dtheta);
    double ds=Math.sin(dtheta);
    for(int i=0;i<ox.length;i++){
      double x0=sma*(cc-eccen);
      double y0=smaY*ss;
      double x=m11*x0+m12*y0;
      double y=m21*x0+m22*y0;
      double z=m31*x0+m32*y0;
      vp.setLocation(x,y,z);
      theta.rotate(vp);
      ox[i]=(int)(rcoef*vp.x)+center.x;
      oy[i]=(int)(rcoef*vp.y)+center.y;
      double cctemp=cc*dc-ss*ds;
      double sstemp=ss*dc+cc*ds;
      cc=cctemp;
      ss=sstemp;
    }
    g.drawPolygon(ox,oy,ox.length);
  }

//--------------------------------------------------------------------//
/**
 * IuWFNg̃Rs[Ԃ
 * @return ̃CX^X̕
 */
  public Object clone(){
    try{
      Planet planet=(Planet)super.clone();
      planet.p=(Point3D)p.clone();
      planet.vp=(ViewPoint3D)vp.clone();
      return planet;
    }
    catch(CloneNotSupportedException ex){
      throw new InternalError();
    }
  }

//--------------------------------------------------------------------//
/**
 * IuWFNg̃nbVR[hlԂ
 * @return IuWFNg̃nbVR[hl
 */
  public int hashCode(){
    long ll[]={Double.doubleToLongBits(sma),
      	       Double.doubleToLongBits(eccen),
      	       Double.doubleToLongBits(lofp),
      	       Double.doubleToLongBits(lofan),
      	       Double.doubleToLongBits(inc),
      	       Double.doubleToLongBits(meanl),
      	       Double.doubleToLongBits(pinv)};
    int bit=name.hashCode();
    bit=31*bit+color.hashCode();
    bit=31*bit+radius;
    for(int i=0;i<ll.length;i++){
      bit=31*bit+(int)(ll[i]^(ll[i]>>>32));
    }
    return bit;
  }

//--------------------------------------------------------------------//
/**
 * IuWFNg̓
 * @param obj rΏۂ̃IuWFNg
 * @return rΏۂ̃IuWFNg <code>Planet</code>
 *         ̃CX^Xœlꍇ true
 */
  public boolean equals(Object obj){
    if(obj instanceof Planet){
      Planet planet=(Planet)obj;
      return (name.equals(planet.name)
      	   && color.equals(planet.color)
      	   && (radius==planet.radius)
      	   && (sma==planet.sma)
      	   && (eccen==planet.eccen)
      	   && (lofp==planet.lofp)
      	   && (lofan==planet.lofan)
      	   && (inc==planet.inc)
      	   && (meanl==planet.meanl)
      	   && (pinv==planet.pinv));
    }
    return false;
  }

//--------------------------------------------------------------------//
/**
 * f[^Ԃ
 * @return ̃IuWFNg̕\
 */
  public String toString(){
    return name;
  }
}
