package planet;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.geom.Point2D;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.Comparator;
import figure.SphericalAngle;
import figure.Vector3D;

//--------------------------------------------------------------------//
/**
 * ff[^f
 */
public class PlanetDataModel{

/** f̕`ʒu̔rIuWFNg */
  protected static final Comparator
    VIEW_COMPARATOR=new PlanetViewComparator();

/**  AU  km ֕ϊW */
  public static final double AU_TO_KM=1.4959787E8;

/** f̋Oa̍ŏl */
  protected static final double MIN_RADIUS=1.0E-8;

/** l */
  protected static final DecimalFormat
    DFORM=new DecimalFormat("#######0.0000");

/** fXg */
  protected PlanetList list;

/** 1AU ̃sNZ */
  protected double rcoef;

/** fEOf臒l (Oa AU) */
  protected double rthres;

/** z̋̍ől */
  protected double maxRadius;

/** _px */
  protected SphericalAngle sa;

/** ڐIuWFNg */
  protected PlanetScale scale;

/** EX */
  protected double epoch=2451545.0;

/**
 * ff[^f\z
 */
  public PlanetDataModel(){
    list=new PlanetList();
    rthres=3.0;
    maxRadius=0.0;
    sa=new SphericalAngle();
  }

//--------------------------------------------------------------------//
/**
 * 
 */
  public void init(){
    list.trimToSize();
    maxRadius=0.0;
    int planetCount=list.getSize();
    for(int i=0;i<planetCount;i++){
      Planet planet=list.get(i);
      planet.setType(rthres);
      planet.setEpoch(epoch);
      maxRadius=Math.max(maxRadius,planet.getMaxRadius());
    }
  }

//--------------------------------------------------------------------//
/**
 * fEOf臒lݒ肷
 * @param threshold 臒l\
 */
  public void setThreshold(String threshold){
    try{
      rthres=Double.parseDouble(threshold);
    }
    catch(NumberFormatException ex){
      ex.printStackTrace();
    }
  }

//--------------------------------------------------------------------//
/**
 * EXݒ肷
 * @param epoch EX
 */
  public void setEpoch(String epoch){
    try{
      this.epoch=Double.parseDouble(epoch);
    }
    catch(NumberFormatException ex){
      ex.printStackTrace();
    }
  }

//--------------------------------------------------------------------//
/**
 * fXgԂ
 * @return fXg
 */
  public PlanetList getList(){
    return list;
  }

//--------------------------------------------------------------------//
/**
 * NbNfԂ
 * @param x }EXʒu <i>x</i> W
 * @param y }EXʒu <i>y</i> W
 * @param viewMode \[h
 * @return }EXʒu\͈͓ɂff[^<br>
 *         Yfꍇ null Ԃ
 */
  public Planet getPlanet(int x,int y,int viewMode){
    int planetCount=list.getSize();
    for(int i=0;i<planetCount;i++){
      Planet planet=list.get(i);
      if(planet.clickPlanet(x,y,viewMode)) return planet;
    }
    return null;
  }

//--------------------------------------------------------------------//
/**
 * fWʒuvZ
 * @param julianDay EX
 */
  public void calculateCoordinates(double julianDay){
    int planetCount=list.getSize();
    for(int i=0;i<planetCount;i++){
      list.get(i).calculateCoordinates(julianDay);
    }
  }

//--------------------------------------------------------------------//
/**
 * f\ʒuݒ肷
 * @param viewMode \[h
 * @param center plS_
 */
  public void setViewPosition(int viewMode,Point2D theta,Point center){
    double viewRadius; // `͈͔a
    if(viewMode==Planet.INNER_PLANET){
      viewRadius=Math.max(rthres,MIN_RADIUS);
    }
    else{
      viewRadius=Math.max(maxRadius,MIN_RADIUS);
    }
    rcoef=0.9*Math.min(center.x,center.y)/viewRadius;
    sa.set(Math.toRadians(theta.getX()),Math.toRadians(theta.getY()));
    int planetCount=list.getSize();
    for(int i=0;i<planetCount;i++){
      list.get(i).setViewPosition(center,rcoef,sa);
    }
    scale=PlanetScale.getPlanetScale(rcoef,viewRadius);
  }

//--------------------------------------------------------------------//
/**
 * fԋ\Ԃ
 * @param pointer f̃|C^
 * @return fԋ\i[z
 */
  public String[] getDistanceStrings(PlanetList pointer){
    if(pointer.getSize()>=2){
      Planet p1=pointer.get(0);
      Planet p2=pointer.get(1);
      Vector3D dp=new Vector3D(p1.getLocation(),p2.getLocation());
      double distAU=dp.magnitude();
      String line[]={"form "+p1.getName(),
      	      	     "to "+p2.getName(),
      	      	     "  "+DFORM.format(distAU)+"AU",
      	      	     "= "+(long)(AU_TO_KM*distAU)+"km"};
      return line;
    }
    else{
      String strings[]={""};
      return strings;
    }
  }

//--------------------------------------------------------------------//
/**
 * f\FԂ
 * @param size z̑傫
 * @param pointer f̃|C^
 * @param defaultColor ftHg\F
 * @return f\Fi[z
 */
  public Color[] getColors(int size,PlanetList pointer,
      	      	      	   Color defaultColor){
    Color colors[]=new Color[size];
    Arrays.fill(colors,defaultColor);
    int psize=pointer.getSize();
    for(int i=0;i<psize;i++){
      colors[i]=pointer.get(i).getColor();
    }
    return colors;
  }

//--------------------------------------------------------------------//
/**
 * f`悷
 * @param g `悷OtBbNXReLXg
 * @param viewMode \[h
 * @param center plS_
 * @param orbitColor O`F
 */
  public void drawPlanet(Graphics g,int viewMode,Point center,
      	      	      	 Color orbitColor){
    list.sort(VIEW_COMPARATOR);
    int planetCount=list.getSize();
    for(int i=0;i<planetCount;i++){
      list.get(i).drawPlanet(g,viewMode,center,rcoef,sa,orbitColor);
    }

    if(scale!=null){
      scale.setSize(g.getFontMetrics());
      scale.setLocation(PlanetScale.DEFAULT_SCALE_SIZE,
      	      	      	2*center.y-scale.height);
      scale.paint(g,orbitColor);
    }
  }
}
