package planet;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import gui.BasicViewCanvas;

//--------------------------------------------------------------------//
/**
 * \LoX
 */
public class PlanetViewCanvas extends BasicViewCanvas{

/** ff[^f */
  protected PlanetDataModel model;

/** \[h */
  public int viewMode;

/** _px */
  protected Point2D theta;

/** _oړx */
  protected double coefX;

/** _܈ړx */
  protected double coefY;

/** f̃|C^ */
  public PlanetList pointer;

/** G[bZ[W */
  protected String message;

/** plSW */
  protected Point centerPoint;

/** tHg */
  protected Font font;

/** wiF */
  protected Color bgColor;

/** OiF */
  protected Color fgColor;

  public PlanetViewCanvas(PlanetDataModel model,String message,Font font,
      	      	      	  Color bgColor,Color fgColor){
    super();
    this.model=model;
    this.font=font;
    this.message=message;
    setColors(bgColor,fgColor);
    theta=new Point2D.Double(-90.0,30.0);
    centerPoint=new Point();
    viewMode=Planet.INNER_PLANET;
    pointer=new PlanetList();
    if(model!=null){
      pointer.add(model.getList().get(0));
      pointer.add(model.getList().get(0));
      pointer.trimToSize();
    }
  }

//--------------------------------------------------------------------//
/**
 * \Fݒ肷
 * @param bgColor wiF
 * @param fgColor OiF
 */
  protected void setColors(Color bgColor,Color fgColor){
    if(bgColor==null){
      this.bgColor=Color.BLACK;
    }
    else{
      this.bgColor=bgColor;
    }
    if(fgColor==null){
      this.fgColor=Color.WHITE;
    }
    else{
      this.fgColor=fgColor;
    }
    setBackground(bgColor);
  }

//--------------------------------------------------------------------//
/**
 * ff[^fԂ
 * @return ff[^f
 */
  public PlanetDataModel getModel(){
    return model;
  }

//--------------------------------------------------------------------//
/**
 * f̃|C^Ԃ
 * @return f̃|C^
 */
  public PlanetList getPointer(){
    return pointer;
  }

//--------------------------------------------------------------------//
/**
 * \[hԂ
 * @return \[h
 */
  public int getViewMode(){
    return viewMode;
  }

//--------------------------------------------------------------------//
/**
 * \[hݒ肷
 * @param viewMode \[h
 */
  public void setViewMode(int viewMode){
    this.viewMode=viewMode;
    repaint();
  }

//--------------------------------------------------------------------//
/**
 * _ʒu]
 * @param dthetaX ]o
 * @param dthetaY ]
 */
  public void rotate(int dthetaX,int dthetaY){
    rotate((double)dthetaX,(double)dthetaY);
  }

//--------------------------------------------------------------------//
/**
 * _ʒu]
 * @param dthetaX ]o
 * @param dthetaY ]
 */
  public void rotate(double dthetaX,double dthetaY){
    double thetaX=theta.getX()+dthetaX;
    double thetaY=theta.getY()+dthetaY;
    while(thetaX>180.0)  thetaX-=360.0;
    while(thetaX<-180.0) thetaX+=360.0;
    thetaY=Math.min(Math.max(thetaY,-90.0),90.0);
    theta.setLocation(thetaX,thetaY);
  }

//--------------------------------------------------------------------//
/**
 * ĕ`悷
 * @param g `悷OtBbNXReLXg
 */
  public void update(Graphics g){
    if(model!=null){
      drawPlanet(bg);
    }
    else{
      drawErrorMessage(bg,message);
    }
    g.drawImage(bgImage,0,0,this);
  }

//--------------------------------------------------------------------//
/**
 * `悷
 * @param g `悷OtBbNXReLXg
 */
  public void paint(Graphics g){
    setPaint();
    if(model!=null){
      drawPlanet(bg);
    }
    else{
      drawErrorMessage(bg,message);
    }
    g.drawImage(bgImage,0,0,this);
  }

//--------------------------------------------------------------------//
/**
 * ʂ̐ݒ
 */
  protected void setPaint(){
    super.setPaint();
    centerPoint.setLocation(panelSize.width/2,panelSize.height/2);
    coefX=72.0/centerPoint.x;
    coefY=72.0/centerPoint.y;
    bg.setFont(font);
  }

//--------------------------------------------------------------------//
/**
 * f`悷
 * @param g `悷OtBbNXReLXg
 */
  protected void drawPlanet(Graphics g){
    g.setColor(bgColor);
    g.fillRect(0,0,panelSize.width,panelSize.height);

    String strings[]=PlanetAnimator.CFORM.getDefaultStrings();
    Rectangle bounds=TextBox.paintStrings
      (g,panelSize,strings,1.0f,1.0f,bgColor,fgColor);

    centerPoint.setLocation(bounds.x/2,panelSize.height/2);
    coefX=0.4*Math.PI/centerPoint.x;
    model.calculateCoordinates(PlanetAnimator.CALENDAR.getJulianDay());
    model.setViewPosition(viewMode,theta,centerPoint);
    model.drawPlanet(g,viewMode,centerPoint,fgColor);

    strings=model.getDistanceStrings(pointer);
    Color color[]=model.getColors(strings.length,pointer,fgColor);
    TextBox.paintStrings
      (g,panelSize,strings,color,1.0f,0.0f,bgColor,fgColor);
  }
}
